import pytest
import quasardb


def test_perf_disable(qdbd_connection):
    qdbd_connection.perf().disable()


def test_perf_enable(qdbd_connection):
    qdbd_connection.perf().enable()


def test_perf_enable_fail_if_performance_is_not_enabled(qdbd_settings):
    cluster = quasardb.Cluster(
        uri=qdbd_settings.get("uri").get("secure"),
        user_name=qdbd_settings.get("security").get("user_name"),
        user_private_key=qdbd_settings.get("security").get("user_private_key"),
        cluster_public_key=qdbd_settings.get("security").get("cluster_public_key"))
    with pytest.raises(quasardb.Error):
        cluster.perf().enable()


def test_perf_get__when_empty(qdbd_connection):
    profiles = qdbd_connection.perf().get()
    assert len(profiles) == 0


def test_perf_get_after_blob_put(qdbd_connection, entry_name, random_string):
    qdbd_connection.perf().enable()
    qdbd_connection.blob(entry_name).put(random_string)
    qdbd_connection.blob(entry_name).get()

    profiles = qdbd_connection.perf().get()
    assert len(profiles) == 2
    assert profiles[0][0] == "blob.put"
    assert profiles[1][0] == "common.get"


def test_perf_clear_after_blob_put(qdbd_connection, entry_name, random_string):
    qdbd_connection.perf().enable()
    qdbd_connection.blob(entry_name).put(random_string)

    profiles = qdbd_connection.perf().get()
    assert len(profiles) != 0

    qdbd_connection.perf().clear()
    profiles = qdbd_connection.perf().get()
    assert len(profiles) == 0



import re
import pytest
import quasardb


def test_capi_version_check():
    # This will throw everytime
    # The version check is the most critical step so it is done first.
    # We thus just have to check that the error we get is related to
    # this check to figure how it went.
    with pytest.raises(quasardb.Error, match=r'^((?!C API version).)*$'):
        quasardb.Cluster("_qdb_://invaliduri")



# import-start
import json
import quasardb
import numpy as np
# import-end

def do_something_async_with(x):
    pass

# connect-start
with quasardb.Cluster("qdb://127.0.0.1:2836") as c:
# connect-end
    def secure_connect():
        user_key = {}
        cluster_key = ""

        with open('user_private.key', 'r') as user_key_file:
            user_key = json.load(user_key_file)
        with open('cluster_public.key', 'r') as cluster_key_file:
            cluster_key = cluster_key_file.read()

        # secure-connect-start
        with quasardb.Cluster(uri='qdb://127.0.0.1:2836',
                              user_name=user_key['username'],
                              user_private_key=user_key['secret_key'],
                              cluster_public_key=cluster_key) as scs:
        # secure-connect-end
            pass

    # create-table-start

    # First we acquire a reference to a table (which may or may not yet exist)
    t = c.table("stocks")

    # Initialize our column definitions
    cols = [quasardb.ColumnInfo(quasardb.ColumnType.Double, "open"),
            quasardb.ColumnInfo(quasardb.ColumnType.Double, "close"),
            quasardb.ColumnInfo(quasardb.ColumnType.Int64, "volume")]

    # Now create the table with the default shard size
    t.create(cols)

    # create-table-end

    # tags-start

    t.attach_tag("nasdaq")

    # tags-end

    # batch-insert-start

    # We need to tell the batch inserter which columns we plan to insert. Note
    # how we give it a hint that we expect to insert 2 rows for each of these columns.
    batch_columns = [quasardb.BatchColumnInfo("stocks", "open", 2),
                     quasardb.BatchColumnInfo("stocks", "close", 2),
                     quasardb.BatchColumnInfo("stocks", "volume", 2)]

    # Now that we know which columns we want to insert, we initialize our batch inserter.
    inserter = c.inserter(batch_columns)


    # Insert the first row: to start a new row, we must provide it with a mandatory
    # timestamp that all values for this row will share. QuasarDB will use this timestamp
    # as its primary index.
    #
    # QuasarDB only supports nanosecond timestamps, so we must specifically convert our
    # dates to nanosecond precision.
    inserter.start_row(np.datetime64('2019-02-01', 'ns'))

    # We now set the values for our columns by their relative offsets: column 0 below
    # refers to the first column we provide in the batch_columns variable above.
    inserter.set_double(0, 3.40)
    inserter.set_double(1, 3.50)
    inserter.set_int64(2, 10000)

    # We tell the batch inserter to start a new row before we can set the values for the
    # next row.
    inserter.start_row(np.datetime64('2019-02-02', 'ns'))

    inserter.set_double(0, 3.50)
    inserter.set_double(1, 3.55)
    inserter.set_int64(2, 7500)

    # Now that we're done, we push the buffer as one single operation.
    inserter.push()

    # batch-insert-end


    # bulk-read-start

    # We can initialize a bulk reader based directly from our table. By
    # providing a dict=True parameter, the QuasarDB API will automatically
    # expose our rows as dicts.
    reader = t.reader(dict=True, ranges=[(np.datetime64('2019-02-01', 'ns'), np.datetime64('2019-02-02', 'ns'))])

    # The bulk reader is exposed as a regular Python iterator
    for row in reader:

        # We can access the row locally within our loop:
        print(row)

        # But because the QuasarDB Python API is zero-copy, our row maintains a
        # reference to the underlying data. If we want to keep the row data alive
        # longer than the local scope, you can use row.copy() as follows:
        do_something_async_with(row.copy())

    # bulk-read-end

    # column-insert-start

    # Our API is built on top of numpy, and provides zero-copy integration with native
    # numpy arrays. As such, we first prepare three different arrays for each of our three
    # columns:
    opens = np.array([3.40, 3.50], dtype=np.float64)
    closes = np.array([3.50, 3.55], dtype=np.float64)
    volumes = np.array([10000, 7500], dtype=np.int64)

    # Seperately, we generate a numpy array of timestamps. Since our three columns share
    # the same timestamps, we can reuse this array for all of them, but this is not required.
    timestamps = np.array([np.datetime64('2019-02-01'), np.datetime64('2019-02-02')], dtype='datetime64[ns]')

    # When inserting, we provide the value arrays en timestamp arrays separately.
    t.double_insert("open", timestamps, opens)
    t.double_insert("close", timestamps, closes)
    t.int64_insert("volume", timestamps, volumes)

    # column-insert-end

    # column-get-start

    # We first prepare the intervals we want to select data from, that is, a list of
    # timeranges. An interval is defined as a tuple of start time (inclusive) and end
    # time (exclusive).
    #
    # In this example, we just use a single interval.
    intervals = np.array([(np.datetime64('2019-02-01', 'ns'), np.datetime64('2019-02-02', 'ns'))])

    # As with insertion, our API works with native numpy arrays and returns the results as such.
    (timestamps1, opens) = t.double_get_ranges("open", intervals)
    (timestamps2, closes) = t.double_get_ranges("close", intervals)
    (timestamps3, volumes) = t.int64_get_ranges("volume", intervals)

    # For this specific dataset, timestamps1 == timestamps2 == timestamps3, but
    # this does not necessarily have to be the case.
    np.testing.assert_array_equal(timestamps1, timestamps2)
    np.testing.assert_array_equal(timestamps1, timestamps3)

    # column-get-end

    # query-start

    result = c.query("SELECT SUM(volume) FROM stocks")

    # results is returned as a list of dicts
    for row in result:
        print("row: ", row)


    # Since we only expect one row, we also access it like this:
    aggregate_result = result[0]['sum(volume)']
    print("sum(volume): ", aggregate_result)

    # query-end

    # drop-table-start

    # Use the earlier reference of the table we acquired to remove it:
    t.remove()

    # drop-table-end



import pytest
import quasardb

def test_node_empty_prefix(qdbd_direct_connection):
    res = qdbd_direct_connection.prefix_get("testazeazeaze", 10)
    assert len(res) == 0

def test_node_find_prefixes(qdbd_direct_connection):
    res = qdbd_direct_connection.prefix_get("$qdb.statistics.", 20)
    assert len(res) == 20

def test_node_blob_get(qdbd_direct_connection):
    entry = qdbd_direct_connection.blob("$qdb.statistics.node_id")  
    got = entry.get()
    assert type(got) is bytes and len(got) > 0

def test_node_integer_get(qdbd_direct_connection):
    entry = qdbd_direct_connection.integer("$qdb.statistics.cpu.system")
    got = entry.get()

    assert type(got) is int and got > 0 


import pdoc
import quasardb
import quasardb.pandas as qdbpd

modules = [quasardb.quasardb, qdbpd]  # Public submodules are auto-imported
context = pdoc.Context()

modules = [pdoc.Module(mod, context=context)
           for mod in modules]

pdoc.link_inheritance(context)

def recursive_htmls(mod):
    yield mod.name, mod.html()
    for submod in mod.submodules():
        yield from recursive_htmls(submod)

for mod in modules:
    for module_name, html in recursive_htmls(mod):
        f = open("doc/" + module_name + ".html", 'w')
        f.write(html)
        f.close()



# pylint: disable=C0103,C0111,C0302,W0212

import quasardb
import pytest

import test_batch_inserter as batchlib

def test_set_client_max_in_buf_1(qdbd_connection):
    with pytest.raises(quasardb.Error):
        qdbd_connection.options().set_client_max_in_buf_size(1)


def test_set_client_max_in_buf_1MB(qdbd_connection):
    qdbd_connection.options().set_client_max_in_buf_size(1024 * 1024)
    assert qdbd_connection.options().get_client_max_in_buf_size() == (1024 * 1024)


def test_get_client_max_in_buf(qdbd_connection):
    assert qdbd_connection.options().get_client_max_in_buf_size() > 0


def test_get_cluster_max_in_buf(qdbd_connection):
    assert qdbd_connection.options().get_cluster_max_in_buf_size() > 0


def test_client_query_buf_size_error(qdbd_connection, table, many_intervals):
    # First insert some data
    inserter = qdbd_connection.inserter(
        batchlib._make_inserter_info(table))

    doubles, blobs, strings, integers, timestamps = batchlib._test_with_table(
        inserter,
        table,
        many_intervals,
        batchlib._row_insertion_method,
        batchlib._regular_push)

    res = qdbd_connection.query(
        "select * from " + table.get_name())

    assert len(res) == 10000

    qdbd_connection.options().set_client_max_in_buf_size(1024)

    with pytest.raises(quasardb.InputBufferTooSmallError, match=r'consider increasing the buffer size'):
        qdbd_connection.query(
            "select * from " + table.get_name())



# pylint: disable=C0103,C0111,C0302,W0212


def test_get_entries(blob_entry, random_blob, tag_name):
    blob_entry.put(random_blob)

    tags = blob_entry.get_tags()
    assert len(tags) == 0

    assert blob_entry.has_tag(tag_name) == False
    assert blob_entry.attach_tag(tag_name)
    assert blob_entry.attach_tag(tag_name) == False

    tags = blob_entry.get_tags()
    assert len(tags) == 1
    assert tags[0] == tag_name

    assert blob_entry.has_tag(tag_name)
    assert blob_entry.detach_tag(tag_name)
    assert blob_entry.detach_tag(tag_name) == False

    tags = blob_entry.get_tags()
    assert len(tags) == 0


def test_tag_multiple(blob_entry, random_blob, tag_names):
    blob_entry.put(random_blob)

    tags = blob_entry.get_tags()
    assert len(tags) == 0

    blob_entry.attach_tags(tag_names)

    assert sorted(blob_entry.get_tags()) == tag_names

    blob_entry.detach_tags(tag_names)

    tags = blob_entry.get_tags()
    assert len(tags) == 0



# pylint: disable=C0103,C0111,C0302,W0212
import datetime
import pytest
import quasardb
import sys
import os

sys.path.append(os.path.join(os.path.split(__file__)[0], '..', 'examples/'))

def test_tutorial_python():
    import tutorial.python

def test_tutorial_pandas():
    import tutorial.pandas



# Copyright (c) 2009-2020, quasardb SAS. All rights reserved.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of quasardb nor the names of its contributors may
#      be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY QUASARDB AND CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from __future__ import print_function
from builtins import int

import os
from socket import gethostname
import sys
import traceback
import random
import time
import datetime
import locale

import numpy as np
import pandas as pd

import quasardb  # pylint: disable=C0413,E0401

def create_ts(q, name, columns):

    ts = q.ts(name)

    try:
        ts.remove()

    except quasardb.Error:
        pass

    ts.create([quasardb.ColumnInfo(quasardb.ColumnType.Double, x) for x in columns])

    return ts

def display_pts(start_time, end_time, points_count):
    elapsed = end_time - start_time
    pts_sec = points_count /elapsed

    print("...loaded {} points in {}s ({} points/sec)".format(points_count, elapsed, pts_sec))

def main(quasardb_uri, ts_name, csv_file):

    print("Loading CSV: ", csv_file)
    start_time = time.time()
    series = pd.read_csv(csv_file, header=0, parse_dates = {'Timestamp' : ['Date', 'Time']}, index_col = 'Timestamp')

    display_pts(start_time, time.time(), series.size)

    print("Uploading to server: ", quasardb_uri)
    q = quasardb.Cluster(quasardb_uri)
    start_time = time.time()
    ts = create_ts(q, ts_name, series.columns.tolist())

    # insert the columns
    for i in range(0, len(series.columns)):
        ts.double_insert(series.columns[i], series.index.values, np.squeeze(series.values[:,i:i+1]))

    display_pts(start_time, time.time(), series.size)

if __name__ == "__main__":

    try:
        if len(sys.argv) != 4:
            print("usage: ", sys.argv[0], " quasardb_uri ts_name csv_file")
            sys.exit(1)

        main(sys.argv[1], sys.argv[2], sys.argv[3])

    except Exception as ex:  # pylint: disable=W0703
        print("An error ocurred:", str(ex))
        traceback.print_exc()



# Copyright (c) 2009-2020, quasardb SAS. All rights reserved.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of quasardb nor the names of its contributors may
#      be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY QUASARDB AND CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from __future__ import print_function

import sys
import traceback
import os

import numpy as np

import quasardb  # pylint: disable=C0413,E0401

def seed_ts(c, name):
    ts = c.ts(name)
    try:
        ts.remove()
    except:
        pass

    ts.create([quasardb.ColumnInfo(quasardb.ColumnType.Int64, "bid"),
                      quasardb.ColumnInfo(quasardb.ColumnType.Int64, "last")])

    # Generate 20 random data points
    date = np.arange(np.datetime64('2018-01-01'), np.datetime64('2018-01-21')).astype('datetime64[ns]')
    values = np.random.random_integers(0, 100, len(date))

    ts.int64_insert("bid", date, np.random.random_integers(0, 100, len(date)))
    ts.int64_insert("last", date, np.random.random_integers(0, 100, len(date)))

    return ts

def main(uri):

    print("Connecting to: ", uri)
    c = quasardb.Cluster(uri=uri)

    ts_name1 = "stocks.apple"
    ts_name2 = "stocks.google"

    ts1 = seed_ts(c, ts_name1)
    ts2 = seed_ts(c, ts_name2)

    # Attach a tag to both of them:
    ts1.attach_tag("nasdaq")
    ts2.attach_tag("nasdaq")

    q = c.query("select sum(bid), count(bid) from find(tag='nasdaq') in range(2018, 2019)")
    res = q.run()

    for k, table_result in res.tables.items():
        print("table:", k)
        for col in table_result:
            print(col.name, ":", col.data)

if __name__ == "__main__":

    try:
        if len(sys.argv) != 2:
            print("usage: ", sys.argv[0], " cluster")
            sys.exit(1)

        main(sys.argv[1])

    except Exception as ex:  # pylint: disable=W0703
        print("An error ocurred:", str(ex))
        traceback.print_exc()



# pylint: disable=missing-module-docstring,missing-function-docstring,not-an-iterable,invalid-name

import numpy as np
import pandas as pd

import quasardb.pandas as qdbpd

ROW_COUNT = 1000


def gen_df(start_time, count):
    idx = pd.date_range(start_time, periods=count, freq='S')

    return pd.DataFrame(data={"the_double": np.random.uniform(-100.0, 100.0, count),
                              "the_int64": np.random.randint(-100, 100, count),
                              "the_blob": np.array([(b"content_" + bytes(item))
                                                    for item in range(count)]),
                              "the_string": np.array([("content_" + str(item))
                                                      for item in range(count)]),
                              "the_ts": np.array([
                                  (start_time + np.timedelta64(i, 's'))
                                  for i in range(count)]).astype('datetime64[ns]')},
                        index=idx)


def gen_series(start_time, count):
    idx = pd.date_range(start_time, periods=count, freq='S')

    return {"the_double": pd.Series(np.random.uniform(-100.0, 100.0, count),
                                    index=idx),
            "the_int64": pd.Series(np.random.randint(-100, 100, count),
                                   index=idx),
            "the_blob": pd.Series(np.array([(b"content_" + bytes(item)) for item in range(count)]),
                                  index=idx),
            "the_string": pd.Series(np.array([("content_" + str(item)) for item in range(count)]),
                                    index=idx),
            "the_ts": pd.Series(np.array([(start_time + np.timedelta64(i, 's'))
                                          for i in range(count)]).astype('datetime64[ns]'),
                                index=idx)}


def test_series_read_write(table):
    series = gen_series(np.datetime64('2017-01-01', 'ns'), ROW_COUNT)

    # Insert everything
    for col in series:
        qdbpd.write_series(series[col], table, col)

    # Read everything
    for col in series:
        read_series = qdbpd.read_series(table, col)
        assert isinstance(read_series, pd.core.series.Series)
        assert read_series.size == ROW_COUNT
        np.testing.assert_array_equal(
            read_series.to_numpy(), series[col].to_numpy())


def test_dataframe(qdbd_connection, table):
    df1 = gen_df(np.datetime64('2017-01-01'), ROW_COUNT)
    qdbpd.write_dataframe(df1, qdbd_connection, table)

    df2 = qdbpd.read_dataframe(table)

    assert len(df1.columns) == len(df2.columns)
    for col in df1.columns:
        np.testing.assert_array_equal(df1[col].to_numpy(), df2[col].to_numpy())


def test_dataframe_can_read_columns(qdbd_connection, table):
    df1 = gen_df(np.datetime64('2017-01-01'), ROW_COUNT)
    qdbpd.write_dataframe(df1, qdbd_connection, table)

    df2 = qdbpd.read_dataframe(table, columns=['the_double', 'the_int64'])

    assert len(df1.columns) != len(df2.columns)
    assert len(df2.columns) == 2
    for col in df2.columns:
        np.testing.assert_array_equal(df1[col].to_numpy(), df2[col].to_numpy())


def test_dataframe_can_read_ranges(qdbd_connection, table):
    start = np.datetime64('2017-01-01', 'ns')
    df1 = gen_df(start, 10)
    qdbpd.write_dataframe(df1, qdbd_connection, table)

    first_range = (start, start + np.timedelta64(1, 's'))
    second_range = (start + np.timedelta64(1, 's'),
                    start + np.timedelta64(2, 's'))

    df2 = qdbpd.read_dataframe(table)
    df3 = qdbpd.read_dataframe(table, ranges=[first_range])
    df4 = qdbpd.read_dataframe(table, ranges=[first_range, second_range])

    assert df2.shape[0] == 10
    assert df3.shape[0] == 1
    assert df4.shape[0] == 2


def test_write_dataframe(qdbd_connection, table):
    # Ensures that we can do a full-circle write and read of a dataframe
    df1 = gen_df(np.datetime64('2017-01-01'), ROW_COUNT)
    qdbpd.write_dataframe(df1, qdbd_connection, table, chunk_size=4)

    df2 = qdbpd.read_dataframe(table)

    assert len(df1.columns) == len(df2.columns)
    for col in df1.columns:
        np.testing.assert_array_equal(df1[col].to_numpy(), df2[col].to_numpy())


def test_write_dataframe_create_table(qdbd_connection, entry_name):
    table = qdbd_connection.ts(entry_name)
    df1 = gen_df(np.datetime64('2017-01-01'), ROW_COUNT)
    qdbpd.write_dataframe(df1, qdbd_connection, table, create=True, blobs='the_blob')

    df2 = qdbpd.read_dataframe(table)

    assert len(df1.columns) == len(df2.columns)
    for col in df1.columns:
        np.testing.assert_array_equal(df1[col].to_numpy(), df2[col].to_numpy())


def test_write_dataframe_create_table_twice(qdbd_connection, table):
    df1 = gen_df(np.datetime64('2017-01-01'), ROW_COUNT)
    qdbpd.write_dataframe(df1, qdbd_connection, table, create=True)


def check_equal(expected, actual):
    if np.isnan(expected):
        assert np.isnan(actual)
    else:
        assert expected == actual


def test_dataframe_read_fast_is_unordered(qdbd_connection, table):
    # As of now, when reading a dataframe fast, when it contains null values,
    # rows are not guaranteed to be ordered; they might be matched to the
    # wrong rows.
    #
    # This will be fixed when the reader supports pinned columns, so we can
    # read the data fast (as numpy arrays) and at the same time keep ordering.

    df1 = gen_df(np.datetime64('2017-01-01'), 2)
    df2 = gen_df(np.datetime64('2017-01-01'), 2)

    ts1 = np.datetime64('2017-01-01 00:00:00', 'ns')
    ts2 = np.datetime64('2017-01-01 00:00:01', 'ns')

    # Now, we set the wrong value for a first row in df1, and for
    # a second row in df2
    df1.at[ts1, 'the_double'] = None
    df2.at[ts2, 'the_double'] = None

    df3 = pd.concat([df1, df2]).sort_index()
    qdbpd.write_dataframe(df3, qdbd_connection, table)

    df4 = qdbpd.read_dataframe(table)

    assert len(df3.columns) == len(df4.columns)
    for col in df3.columns:
        expected = True
        if col == 'the_double':
            expected = False

        assert np.array_equal(df3[col].to_numpy(),
                              df4[col].to_numpy()) == expected

    df5 = qdbpd.read_dataframe(table, row_index=True)

    assert df5.at[0, 'the_int64'] == df1.at[ts1, 'the_int64']
    assert df5.at[1, 'the_int64'] == df2.at[ts1, 'the_int64']
    assert df5.at[2, 'the_int64'] == df1.at[ts2, 'the_int64']
    assert df5.at[3, 'the_int64'] == df2.at[ts2, 'the_int64']

    # QDB-2418
    check_equal(df5.at[0, 'the_double'], df1.at[ts1, 'the_double'])
    check_equal(df5.at[1, 'the_double'], df2.at[ts1, 'the_double'])
    check_equal(df5.at[2, 'the_double'], df1.at[ts2, 'the_double'])
    check_equal(df5.at[3, 'the_double'], df2.at[ts2, 'the_double'])


def test_query(qdbd_connection, table):
    df1 = gen_df(np.datetime64('2017-01-01'), ROW_COUNT)
    qdbpd.write_dataframe(df1, qdbd_connection, table)

    res = qdbpd.query(qdbd_connection, "SELECT * FROM " +
                      table.get_name(), blobs=['the_blob'])

    for col in df1.columns:
        np.testing.assert_array_equal(df1[col].to_numpy(), res[col].to_numpy())



import re
import pytest
import quasardb
import logging
import time

def test_native_logging_output(blob_entry, random_string, caplog):
    caplog.set_level(logging.DEBUG)

    # Do something that generates native logs
    blob_entry.put(random_string)

    # Wait for QuasarDB to flush the logs
    time.sleep(6)

    # Do something that will flush the logs
    blob_entry.remove()

    # Gather all module names that have generated logs, we expect some stuff from at
    # least quasardb.native which we want to test here
    messages = list(lr.message for lr in caplog.records)
    modules = list(lr.name for lr in caplog.records)

    print("messages: ", messages)
    print("modules: ", modules)

    assert 'quasardb.native' in modules
    assert 'requested clients broker service run' in messages



# Copyright (c) 2009-2020, quasardb SAS. All rights reserved.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of quasardb nor the names of its contributors may
#      be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY QUASARDB AND CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from __future__ import print_function
from builtins import range as xrange, int

import os
from socket import gethostname
import sys
import traceback
import random
import numpy as np
import time
import locale

import quasardb  # pylint: disable=C0413,E0401

COLUMN1_NAME = "value"
COLUMN2_NAME = "location"

cities = ["Tokyo",
          "New-York",
          "Sao Paulo",
          "Seoul",
          "Mexico",
          "Osaka",
          "Manila",
          "Mumbai",
          "Delhi",
          "Jakarta",
          "Lagos",
          "Kolkata",
          "Cairo",
          "Los Angeles",
          "Buenos Aires",
          "Rio de Janeiro",
          "Moscow",
          "Shanghai",
          "Karachi",
          "Paris"]

def time_execution(str, f, *args):
    print("     - ", str, end='')

    start_time = time.time()
    res = f(*args)
    end_time = time.time()

    print(" [duration: {}s]".format(end_time - start_time))

    return res

def gen_ts_name():
    return "test.{}.{}.{}".format(gethostname(), os.getpid(), random.randint(0, 100000))

def create_ts(q, name):
    ts = q.ts(name)

    try:
        ts.remove()

    except quasardb.Error:
        pass

    ts.create([quasardb.ColumnInfo(quasardb.ColumnType.Double, COLUMN1_NAME), quasardb.ColumnInfo(quasardb.ColumnType.Blob, COLUMN2_NAME)])

    return ts

def generate_points(points_count):
    dates = np.arange(np.datetime64('2017-01-01'), np.datetime64('2017-01-01') + np.timedelta64(points_count, 'm')).astype('datetime64[ns]')
    values = np.random.uniform(-10.0, 40.0, len(dates))

    return (dates, values)

def generate_cities(dates, values):
    res = []

    for i in range(0, len(values)):
      res.append(random.choice(cities))

    return np.array(res)

def insert(q, ts_name, points_count):
    ts = time_execution("Creating a time series of name {}".format(ts_name), create_ts, q, ts_name)
    (dates, values) = time_execution("Generating {:,} points".format(points_count), generate_points, points_count)
    cities = generate_cities(dates, values)

    time_execution("Inserting {:,} temperature points into {}".format(points_count, ts_name), ts.double_insert, COLUMN1_NAME, dates, values)
    time_execution("Inserting {:,} locations into {}".format(points_count, ts_name), ts.blob_insert, COLUMN2_NAME, dates, cities)

def main(quasardb_uri, ts_name, points_count):

    print("Connecting to: ", quasardb_uri)
    q = quasardb.Cluster(uri=quasardb_uri)

    print(" *** Inserting {:,} into {}".format(points_count, quasardb_uri))
    insert(q, ts_name, points_count)

if __name__ == "__main__":

    try:
        if len(sys.argv) != 4:
            print("usage: ", sys.argv[0], " quasardb_uri ts_name points_count")
            sys.exit(1)

        main(sys.argv[1], sys.argv[2], int(sys.argv[3]))

    except Exception as ex:  # pylint: disable=W0703
        print("An error ocurred:", str(ex))
        traceback.print_exc()



# pylint: disable=C0103,C0111,C0302,W0212
from builtins import range as xrange  # pylint: disable=W0622
import pytest
import quasardb

def test_put(integer_entry, random_integer):
    integer_entry.put(random_integer)

def test_get(integer_entry, random_integer):
    integer_entry.put(random_integer)
    got = integer_entry.get()

    assert random_integer == got

def test_update(integer_entry, random_integer):
    integer_entry.put(random_integer)
    got = integer_entry.get()

    assert random_integer == got

    new_integer = 42
    integer_entry.update(new_integer)
    got = integer_entry.get()

    assert 42 == got

def test_add(integer_entry, random_integer):
    integer_entry.put(random_integer)
    got = integer_entry.get()

    assert random_integer == got

    integer_entry.add(42)
    got = integer_entry.get()

    assert (random_integer + 42) == got
